@@ -1,9 +1,9 @@
-import React, { FC } from 'react';
+import React from 'react';
 import { connect, ConnectedProps } from 'react-redux';
 import { useEffectOnce } from 'react-use';
 
 import { config } from '@grafana/runtime';
-import { Button, HorizontalGroup } from '@grafana/ui';
+import { Button, Stack } from '@grafana/ui';
 import { Page } from 'app/core/components/Page/Page';
 import { StoreState, UserOrg } from 'app/types';
 
@@ -35,7 +35,7 @@ const connector = connect(mapStateToProps, mapDispatchToProps);
 
 type Props = ConnectedProps<typeof connector>;
 
-export const SelectOrgPage: FC<Props> = ({ setUserOrganization, getUserOrganizations, userOrgs }) => {
+export const SelectOrgPage = ({ setUserOrganization, getUserOrganizations, userOrgs }: Props) => {
   const setUserOrg = async (org: UserOrg) => {
     await setUserOrganization(org.orgId);
     window.location.href = config.appSubUrl + '/';
@@ -53,14 +53,14 @@ export const SelectOrgPage: FC<Props> = ({ setUserOrganization, getUserOrganizat
             You have been invited to another organization! Please select which organization that you want to use right
             now. You can change this later at any time.
           </p>
-          <HorizontalGroup wrap>
+          <Stack wrap="wrap">
             {userOrgs &&
               userOrgs.map((org) => (
                 <Button key={org.orgId} icon="signin" onClick={() => setUserOrg(org)}>
                   {org.name}
                 </Button>
               ))}
-          </HorizontalGroup>
+          </Stack>
         </div>
       </Page.Contents>
     </Page>
